@@ -9,7 +9,7 @@
 
   Return codes:
  
-    0             Success.
+    0             Success.  Problem solved to optimality.
     1             Success.  The problem is primal infeasibile, and we
                             have a certificate.
     2             Success.  The problem is dual infeasible, and we have 
@@ -23,7 +23,7 @@
     7             Failure: Lack of progress
     8             Failure: X, Z, or O was singular.
     9             Failure: Detected NaN or Inf values.
-
+   10             Failure: Stopped by QUIT, KILL, TERM, SIGXCPU signal
 
   Notes on data storage:  All "2-d" arrays are stored in Fortran style,
   column major order.  macros in index.h are used to handle indexing into
@@ -114,7 +114,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
   double relgap;
   double mu=1.0e30;
   double muplus;
-  double oldmu;
   double muk;
   double gamma;
   double alphap,alphap1;
@@ -130,10 +129,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
   double bestrelpinfeas;
   double limreldinfeas;
   double bestreldinfeas;
-  double maxrelinfeas=1.0e100;
-  double oldmaxrelinfeas=1.0e100;
-  double newpobj;
-  double newdobj;
   int iter=0;
   int m;
   int ret;
@@ -144,7 +139,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
   int retries=0;
   int retcode=0;
   double bestmeas;
-  int lastimprove=0;
   double diagnrm;
   double diagfact=0.0;
   double diagadd;
@@ -215,7 +209,7 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
    */
 
 #define BASIZE 100
-  double bestarray[BASIZE+1];
+    double bestarray[BASIZE+1];
 
   /*
    * Used in checking whether the primal-dual affine step gives us a new
@@ -224,6 +218,13 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
   
   double affgap,affpobj,affdobj,affrelgap,affrelpinfeas,affreldinfeas;
 
+  /*
+   * Indicate the version of CSDP.
+   */
+
+  if (printlevel >= 1)
+    printf("CSDP 6.2.0\n");
+  
   /*
    * Precompute norms of a and C, so that we don't keep doing this 
    * again and again.
@@ -374,17 +375,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
   if (reldinfeas < parameters.atytol)
     probdfeas=1;
 
-
-  oldmaxrelinfeas=maxrelinfeas;
-  if (relpinfeas > reldinfeas)
-    {
-      maxrelinfeas=relpinfeas;
-    }
-  else
-    {
-      maxrelinfeas=reldinfeas;
-    };
-
   /*
    * Record this solution as the best solution.
    */
@@ -421,10 +411,26 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	    * Call the user exit routine, and let the user stop the process
 	    * if he wants to.
 	    */
-	   if (user_exit(n,k,C,a,*dobj,*pobj,constant_offset,constraints,
-			 X,y,Z,parameters) == 1)
+           
+           ret=user_exit(n,k,C,a,*dobj,*pobj,constant_offset,constraints,
+			 X,y,Z,parameters);
+	   if (ret >= 1)
 	     {
-	       return(0);
+               if (ret==1)
+                 {
+                   /*
+                    * Received SIGTERM, SIGQUIT, SIGXCPU, ...
+                    */
+                   return(10);
+                 }
+               else
+                 {
+                   /*
+                    * User exit was good enough, returning success.
+                    */
+                   return(0);                   
+                 };
+
 	     }
 
 	   bestarray[iter % BASIZE]=bestmeas;
@@ -675,10 +681,10 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 
 	   for (i=1; i<=k-1; i++)
 	     for (j=i; j<=k; j++)
-	       O[ijtok(j,i,ldam)]=O[ijtok(i,j,ldam)];
+	       O[ijtok(j,i, (long int) ldam)]=O[ijtok(i,j, (long int) ldam)];
 	   
 	   for (i=1; i<=k; i++)
-	     diagO[i]=O[ijtok(i,i,ldam)];
+	     diagO[i]=O[ijtok(i,i,(long int) ldam)];
 
 	   mindiag=1.0e30;
 
@@ -735,14 +741,14 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	     printf("diagnrm is %e, adding diagadd %e \n",diagnrm,diagadd);
 
 	   for (i=1; i<=k; i++)
-	     O[ijtok(i,i,ldam)] += diagadd;
+	     O[ijtok(i,i, (long int) ldam)] += diagadd;
 
 	   /*
 	    * Scale the O matrix.
 	    */
 
 	   for (i=1; i<=k; i++)
-	     workvec8[i]=1.0/sqrt(O[ijtok(i,i,ldam)]);
+	     workvec8[i]=1.0/sqrt(O[ijtok(i,i, (long int) ldam)]);
 
 	   for (i=1; i<=k; i++)
 	     {
@@ -753,7 +759,7 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 #pragma omp parallel for schedule(dynamic,64) default(none) shared(O,ldam,k,workvec8) private(i,j)
 	  for (j=1; j<=k; j++)
 	     for (i=1; i<=j; i++)
-	       O[ijtok(i,j,ldam)]=O[ijtok(i,j,ldam)]*(workvec8[i]*workvec8[j]);
+	       O[ijtok(i,j, (long int) ldam)]=O[ijtok(i,j, (long int) ldam)]*(workvec8[i]*workvec8[j]);
 
 	   /*
 	     Next, compute the cholesky factorization of the system matrix.
@@ -798,9 +804,9 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 #pragma omp parallel for schedule(dynamic,64) private(i,j) shared(O,k,ldam)
 		   for (i=1; i<=k-1; i++)
 		     for (j=i; j<=k; j++)
-		       O[ijtok(i,j,ldam)]=O[ijtok(j,i,ldam)];
+		       O[ijtok(i,j, (long int) ldam)]=O[ijtok(j,i, (long int) ldam)];
 		   for (i=1; i<=k; i++)
-		     O[ijtok(i,i,ldam)]=diagO[i];
+		     O[ijtok(i,i,(long int) ldam)]=diagO[i];
 		   goto RETRYFACTOR;
 		 }
 	       else
@@ -1097,8 +1103,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 				workvec5,workvec6,mystepfrac,1.0,
 				printlevel);
 
-	   oldmu=mu;
-
 	   /* Here, work1 holds X+alphap1*dX, work2=Z+alphad1*dZ */
 
 	   addscaledmat(X,alphap1,dX,work1);
@@ -1155,19 +1159,30 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	       /*
 		* run user exit to check if the affine solution is good enough
 		*/
-	       if (user_exit(n,k,C,a,affdobj,affpobj,constant_offset,constraints,
-			     work1,workvec1,work2,parameters) == 1) 
-		 {
-		   *dobj=affdobj;
-		   *pobj=affpobj;
-		   copy_mat(work1,X);
-		   copy_mat(work2,Z);
-		   for(i=1; i<=k; i++)
-		     y[i]=workvec1[i];
-		   if(printlevel>=3)
-		     printf("Affine step good enough, exiting\n");
-		   return(0);
-		 };
+               ret=user_exit(n,k,C,a,affdobj,affpobj,constant_offset,constraints,
+			     work1,workvec1,work2,parameters);
+	       if (ret >= 1)
+                 {
+                   if (ret == 1)
+                     {
+                       /*
+                        * Received SIGTERM, SIGQUIT, ...
+                        */
+                       return(10);
+                     }
+                   else
+                     {
+                       *dobj=affdobj;
+                       *pobj=affpobj;
+                       copy_mat(work1,X);
+                       copy_mat(work2,Z);
+                       for(i=1; i<=k; i++)
+                         y[i]=workvec1[i];
+                       if(printlevel>=3)
+                         printf("Affine step good enough, exiting\n");
+                       return(0);
+                     };
+                 };
 
 	       if (parameters.usexzgap==0)
 		 {
@@ -1200,8 +1215,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 		   (affrelpinfeas/parameters.axtol <bestmeas) &&
 		   (affreldinfeas/parameters.atytol <bestmeas))
 		 {
-		   lastimprove=iter;
-		   
 		   bestmeas=affrelgap/parameters.objtol;
 		   if (affrelpinfeas/parameters.axtol > bestmeas)
 		     bestmeas=affrelpinfeas/parameters.axtol;
@@ -1219,8 +1232,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	       if ((ispfeasprob==1) &&
 		   (affrelpinfeas/parameters.axtol < bestmeas))
 		 {
-		   lastimprove=iter;
-		   
 		   bestmeas=affrelpinfeas/parameters.axtol;
 		   
 		   store_packed(work1,bestx);
@@ -1237,8 +1248,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	       if ((isdfeasprob==1) &&
 		   (affreldinfeas/parameters.atytol <bestmeas))
 		 {
-		   lastimprove=iter;
-		   
 		   bestmeas=affreldinfeas/parameters.atytol;
 		   
 		   zero_mat(work3);
@@ -1630,13 +1639,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 			     workvec5,workvec6,mystepfrac,1.0,
 			     printlevel);
 
-
-	   /*
-	    * Compute the objective value of the new solution.
-	    */
-
-	   newpobj=calc_pobj(C,work1,constant_offset);
-
 	   alphad=linesearch(n,dZ,work1,work2,work3,cholzinv,workvec4,
 			     workvec5,workvec6,mystepfrac,1.0,
 			     printlevel);
@@ -1649,12 +1651,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	   for (i=1; i<=k; i++)
 	     workvec1[i]=y[i]+alphad*dy[i];
 
-	   /*
-	    * Calculate the prospective new dual objective.
-	    */
-
-	   newdobj=calc_dobj(k,a,workvec1,constant_offset);
-
 	   /*
 	    * Check on the feasibility of the new solutions.  If they're
 	    * worse, and the old solution was feasible, then don't take the
@@ -1714,7 +1710,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	       alphap=0.80*alphap;
 	       addscaledmat(X,alphap,dX,work1);
 	       newrelpinfeas=pinfeas(k,constraints,work1,a,workvec1);
-	       newpobj=calc_pobj(C,work1,constant_offset);
 	       i=i+1;
 
 	       if (i>20)
@@ -1730,9 +1725,9 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 		       diagfact=diagfact*10.0;
 		       for (i=1; i<=k-1; i++)
 			 for (j=i; j<=k; j++)
-			   O[ijtok(i,j,ldam)]=O[ijtok(j,i,ldam)];
+			   O[ijtok(i,j, (long int) ldam)]=O[ijtok(j,i, (long int) ldam)];
 		       for (i=1; i<=k; i++)
-			 O[ijtok(i,i,ldam)]=diagO[i];
+			 O[ijtok(i,i,(long int) ldam)]=diagO[i];
 		       goto RETRYFACTOR;
 		     }
 		   else
@@ -1804,7 +1799,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	       
 	       newreldinfeas=dinfeas(k,C,constraints,workvec1,work1,
 				     work2);
-	       newdobj=calc_dobj(k,a,workvec1,constant_offset);
 	       i=i+1;
 	       if (i>15)
 		 {
@@ -1819,9 +1813,9 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 		       diagfact=diagfact*10.0;
 		       for (i=1; i<=k-1; i++)
 			 for (j=i; j<=k; j++)
-			   O[ijtok(i,j,ldam)]=O[ijtok(j,i,ldam)];
+			   O[ijtok(i,j,(long int) ldam)]=O[ijtok(j,i,(long int) ldam)];
 		       for (i=1; i<=k; i++)
-			 O[ijtok(i,i,ldam)]=diagO[i];
+			 O[ijtok(i,i,(long int) ldam)]=diagO[i];
 		       goto RETRYFACTOR;
 		     }
 		   else
@@ -1864,9 +1858,9 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 		   diagfact=diagfact*10.0;
 		   for (i=1; i<=k-1; i++)
 		     for (j=i; j<=k; j++)
-		       O[ijtok(i,j,ldam)]=O[ijtok(j,i,ldam)];
+		       O[ijtok(i,j,(long int) ldam)]=O[ijtok(j,i,(long int) ldam)];
 		   for (i=1; i<=k; i++)
-		     O[ijtok(i,i,ldam)]=diagO[i];
+		     O[ijtok(i,i,(long int) ldam)]=diagO[i];
 		   goto RETRYFACTOR;
 		 }
 	       else
@@ -1878,7 +1872,7 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 		    * Tighten up the solution as much as possible.
 		    */
 		   
-		   retcode=9;
+		   retcode=7;
 		   goto RETURNBEST;
 		   
 		 };
@@ -1889,13 +1883,11 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	    */
 
 	   addscaledmat(X,alphap,dX,work1);
-	   newpobj=calc_pobj(C,work1,constant_offset);
 	   newrelpinfeas=pinfeas(k,constraints,work1,a,workvec1);
 
 	   addscaledmat(Z,alphad,dZ,work1);
 	   for (i=1; i<=k; i++)
 	     workvec1[i]=y[i]+alphad*dy[i];
-	   newdobj=calc_dobj(k,a,workvec1,constant_offset);
 	   
 	   newreldinfeas=dinfeas(k,C,constraints,workvec1,work1,work2);
 
@@ -1989,13 +1981,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	   for (i=1; i<=k; i++)
 	     y[i]=y[i]+alphad*dy[i];
 
-	   /*
-	    *  Update the objectives.
-	    */
-
-	   newdobj=calc_dobj(k,a,y,constant_offset);
-	   newpobj=calc_pobj(C,X,constant_offset);
-
 	   /*
 	     Recompute the objective function values.
 	   */
@@ -2034,16 +2019,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	   if (reldinfeas < parameters.atytol)
 	     probdfeas=1;
 
-	   oldmaxrelinfeas=maxrelinfeas;
-	   if (relpinfeas > reldinfeas)
-	     {
-	       maxrelinfeas=relpinfeas;
-	     }
-	   else
-	     {
-	       maxrelinfeas=reldinfeas;
-	     };
-
 	   /*
 	    * Make sure that the objective value hasn't gone crazy.
 	    *
@@ -2054,7 +2029,7 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 
 	   if ((gap) != (gap))
 	     {
-	       retcode=12;
+	       retcode=9;
 	       goto RETURNBEST;
 	     };
 
@@ -2065,7 +2040,7 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 
 	   if ((gap > 1.0e100) || (gap < -1.0e100))
 	     {
-	       retcode=12;
+	       retcode=9;
 	       goto RETURNBEST;
 	     };
 
@@ -2079,8 +2054,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	       (ispfeasprob==0) && (isdfeasprob==0) &&
 	       (reldinfeas/parameters.atytol <bestmeas))
 	     {
-	       lastimprove=iter;
-
 	       bestmeas=relgap/parameters.objtol;
 	       if (relpinfeas/parameters.axtol > bestmeas)
 		 bestmeas=relpinfeas/parameters.axtol;
@@ -2098,8 +2071,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	   if ((ispfeasprob==1) &&
 	       (relpinfeas/parameters.axtol <bestmeas))
 	     {
-	       lastimprove=iter;
-
 	       bestmeas=relpinfeas/parameters.axtol;
 
 	       store_packed(X,bestx);
@@ -2116,8 +2087,6 @@ int sdp(n,k,C,a,constant_offset,constraints,byblocks,fill,X,y,Z,cholxinv,
 	   if ((isdfeasprob==1) &&
 	       (reldinfeas/parameters.atytol <bestmeas))
 	     {
-	       lastimprove=iter;
-
 	       bestmeas=reldinfeas/parameters.atytol;
 
 	       zero_mat(work3);
